# Generated by Django 4.2.11 on 2024-04-25 11:48

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("projectroles", "0028_populate_finder_role"),
    ]

    operations = [
        migrations.CreateModel(
            name="SODARUserAdditionalEmail",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("email", models.EmailField(help_text="Email address", max_length=254)),
                (
                    "verified",
                    models.BooleanField(
                        default=False, help_text="Email verification status"
                    ),
                ),
                (
                    "secret",
                    models.CharField(
                        help_text="Secret token for email verification",
                        max_length=255,
                        unique=True,
                    ),
                ),
                (
                    "date_created",
                    models.DateTimeField(
                        auto_now_add=True, help_text="DateTime of creation"
                    ),
                ),
                (
                    "date_modified",
                    models.DateTimeField(
                        auto_now=True, help_text="DateTime of last modification"
                    ),
                ),
                (
                    "sodar_uuid",
                    models.UUIDField(
                        default=uuid.uuid4,
                        help_text="SODARUserAdditionalEmail SODAR UUID",
                        unique=True,
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        help_text="User for whom the email is assigned",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="additional_emails",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "ordering": ["user__username", "email"],
                "unique_together": {("user", "email")},
            },
        ),
    ]



"""Django checks for the projectroles app"""

from django.conf import settings
from django.core.checks import Warning, register


# Local constants
W001_SETTINGS = [
    'ENABLE_LDAP',
    'ENABLE_OIDC',
    'PROJECTROLES_ALLOW_ANONYMOUS',
    'PROJECTROLES_ALLOW_LOCAL_USERS',
    'PROJECTROLES_KIOSK_MODE',
]
W001_MSG = (
    'No authentication methods enabled, only superusers can access the site. '
    'Set one or more of the following: {}'.format(', '.join(W001_SETTINGS))
)
W001 = Warning(W001_MSG, obj=settings, id='projectroles.W001')


@register()
def check_auth_methods(app_configs, **kwargs):
    """
    Check for enabled authentication schemes. Raise error if no users other than
    superusers are able to log in with the current settings).
    """
    ret = []
    if not any([getattr(settings, a, False) for a in W001_SETTINGS]):
        ret.append(W001)
    return ret



# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-04-11 16:00
from __future__ import unicode_literals

from django.db import migrations


def populate_roles(apps, schema_editor):
    Role = apps.get_model('projectroles', 'Role')

    def save_role(name, description):
        role = Role(name=name, description=description)
        role.save()

    # Owner
    save_role(
        'project owner',
        'The project owner has full access to project data; rights to add, '
        'modify and remove project members; right to assign a project '
        'delegate. Each project must have exactly one owner.')

    # Delegate
    save_role(
        'project delegate',
        'The project delegate has all the rights of a project owner with the '
        'exception of assigning a delegate. A maximum of one delegate can be '
        'set per project. A delegate role can be set by project owner.')

    # Contributor
    save_role(
        'project contributor',
        'A project member with ability to view and add project data. Can edit '
        'their own data.')

    # Guest
    save_role(
        'project guest',
        'Read-only access to a project. Can view data in project, can not add '
        'or edit.')


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0002_auto_20180411_1758'),
    ]

    operations = [
        migrations.RunPython(populate_roles),
    ]



"""Serializers for the sodarcache app REST API"""

# Projectroles dependency
from projectroles.serializers import (
    SODARProjectModelSerializer,
    SODARUserSerializer,
)

from sodarcache.models import JSONCacheItem


class JSONCacheItemSerializer(SODARProjectModelSerializer):
    """Serializer for the JSONCacheItem model"""

    user = SODARUserSerializer(read_only=True)

    class Meta:
        model = JSONCacheItem
        fields = [
            'project',
            'app_name',
            'name',
            'date_modified',
            'user',
            'data',
        ]  # NOTE: No UUID returned as these should not be referred to directly
        read_only_fields = ['date_modified']



# Generated by Django 4.2.13 on 2024-06-20 14:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("projectroles", "0031_remotesite_owner_modifiable"),
    ]

    operations = [
        migrations.AlterField(
            model_name="appsetting",
            name="value",
            field=models.CharField(
                blank=True, help_text="Value of the setting", null=True
            ),
        ),
    ]



"""Tests for management commands in the projectroles Django app"""

import os
from tempfile import NamedTemporaryFile

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core import mail
from django.core.management import call_command
from django.forms.models import model_to_dict
from django.test import override_settings
from djangoplugins.models import Plugin

from test_plus.test import TestCase

# Timeline dependency
from timeline.models import TimelineEvent

from projectroles.management.commands.addremotesite import (
    Command as AddRemoteSiteCommand,
)
from projectroles.management.commands.batchupdateroles import (
    Command as BatchUpdateRolesCommand,
)
from projectroles.management.commands.cleanappsettings import (
    START_MSG,
    END_MSG,
    DEFINITION_NOT_FOUND_MSG,
    ALLOWED_TYPES_MSG,
    DELETE_PREFIX_MSG,
    DELETE_PROJECT_TYPE_MSG,
    DELETE_SCOPE_MSG,
)
from projectroles.management.commands.createdevusers import (
    DEV_USER_NAMES,
    DEFAULT_PASSWORD,
)
from projectroles.management.commands.syncgroups import (
    Command as SyncGroupsCommand,
)

from projectroles.models import (
    RoleAssignment,
    ProjectInvite,
    RemoteSite,
    AppSetting,
    SODAR_CONSTANTS,
)
from projectroles.tests.test_models import (
    ProjectMixin,
    RoleMixin,
    RoleAssignmentMixin,
    ProjectInviteMixin,
    AppSettingMixin,
    RemoteSiteMixin,
)
from projectroles.utils import build_secret


User = get_user_model()


# SODAR constants
PROJECT_ROLE_OWNER = SODAR_CONSTANTS['PROJECT_ROLE_OWNER']
PROJECT_ROLE_DELEGATE = SODAR_CONSTANTS['PROJECT_ROLE_DELEGATE']
PROJECT_ROLE_CONTRIBUTOR = SODAR_CONSTANTS['PROJECT_ROLE_CONTRIBUTOR']
PROJECT_ROLE_GUEST = SODAR_CONSTANTS['PROJECT_ROLE_GUEST']
PROJECT_TYPE_CATEGORY = SODAR_CONSTANTS['PROJECT_TYPE_CATEGORY']
PROJECT_TYPE_PROJECT = SODAR_CONSTANTS['PROJECT_TYPE_PROJECT']
SITE_MODE_SOURCE = SODAR_CONSTANTS['SITE_MODE_SOURCE']
SITE_MODE_PEER = SODAR_CONSTANTS['SITE_MODE_PEER']
SITE_MODE_TARGET = SODAR_CONSTANTS['SITE_MODE_TARGET']
SYSTEM_USER_GROUP = SODAR_CONSTANTS['SYSTEM_USER_GROUP']

# Local constants
EXAMPLE_APP_NAME = 'example_project_app'
CLEAN_LOG_PREFIX = 'INFO:projectroles.management.commands.cleanappsettings:'
CUSTOM_PASSWORD = 'custompass'
REMOTE_SITE_NAME = 'Test Site'
REMOTE_SITE_URL = 'https://example.com'
REMOTE_SITE_SECRET = build_secret(32)
LDAP_DOMAIN = 'EXAMPLE'


# Mixins -----------------------------------------------------------------------


class BatchUpdateRolesMixin:
    """
    Helpers for batchupdateroles testing.

    NOTE: May be needed by external repos for testing, do not remove!
    """

    file = None

    def write_file(self, data):
        """Write data to temporary CSV file"""
        if not isinstance(data[0], list):
            data = [data]
        self.file.write(
            bytes('\n'.join([';'.join(r) for r in data]), encoding='utf-8')
        )
        self.file.close()


# Tests ------------------------------------------------------------------------


class TestAddRemoteSite(
    ProjectMixin,
    RoleMixin,
    RoleAssignmentMixin,
    RemoteSiteMixin,
    TestCase,
):
    """Tests for addremotesite command"""

    def setUp(self):
        self.user = self.make_user('superuser')
        self.user.is_superuser = True
        self.user.save()
        self.command = AddRemoteSiteCommand()
        self.options = {
            'name': REMOTE_SITE_NAME,
            'url': REMOTE_SITE_URL,
            'mode': SITE_MODE_TARGET,
            'description': '',
            'secret': REMOTE_SITE_SECRET,
            'user_display': True,
            'owner_modifiable': False,
            'suppress_error': False,
        }

    def test_add(self):
        """Test adding new remote site"""
        tl_kwargs = {'event_name': 'target_site_create'}
        self.assertEqual(RemoteSite.objects.count(), 0)
        self.assertEqual(TimelineEvent.objects.filter(**tl_kwargs).count(), 0)
        self.command.handle(**self.options)

        self.assertEqual(RemoteSite.objects.count(), 1)
        site = RemoteSite.objects.first()
        expected = {
            'id': site.pk,
            'name': REMOTE_SITE_NAME,
            'url': REMOTE_SITE_URL,
            'mode': SITE_MODE_TARGET,
            'description': '',
            'secret': REMOTE_SITE_SECRET,
            'sodar_uuid': site.sodar_uuid,
            'user_display': True,
            'owner_modifiable': False,
        }
        self.assertEqual(model_to_dict(site), expected)
        self.assertEqual(TimelineEvent.objects.filter(**tl_kwargs).count(), 1)

    @override_settings(PROJECTROLES_SITE_MODE=SITE_MODE_TARGET)
    def test_add_source_on_target(self):
        """Test adding source site on target site"""
        tl_kwargs = {'event_name': 'source_site_create'}
        self.options['mode'] = SITE_MODE_SOURCE
        self.assertEqual(RemoteSite.objects.count(), 0)
        self.assertEqual(TimelineEvent.objects.filter(**tl_kwargs).count(), 0)
        self.command.handle(**self.options)
        self.assertEqual(RemoteSite.objects.count(), 1)
        self.assertEqual(TimelineEvent.objects.filter(**tl_kwargs).count(), 1)

    def test_add_source_on_source(self):
        """Test adding source site on source site (should fail)"""
        self.options['mode'] = SITE_MODE_SOURCE
        self.assertEqual(RemoteSite.objects.count(), 0)
        with self.assertRaises(SystemExit) as cm:
            self.command.handle(**self.options)
        self.assertEqual(cm.exception.code, 1)
        self.assertEqual(RemoteSite.objects.count(), 0)

    @override_settings(PROJECTROLES_SITE_MODE=SITE_MODE_TARGET)
    def test_add_target_on_target(self):
        """Test adding target site on target site (should fail)"""
        self.assertEqual(RemoteSite.objects.count(), 0)
        with self.assertRaises(SystemExit) as cm:
            self.command.handle(**self.options)
        self.assertEqual(cm.exception.code, 1)
        self.assertEqual(RemoteSite.objects.count(), 0)

    def test_add_peer_on_source(self):
        """Test adding peer site on source site (should fail)"""
        self.options['mode'] = SITE_MODE_PEER
        self.assertEqual(RemoteSite.objects.count(), 0)
        with self.assertRaises(SystemExit) as cm:
            self.command.handle(**self.options)
        self.assertEqual(cm.exception.code, 1)
        self.assertEqual(RemoteSite.objects.count(), 0)

    @override_settings(PROJECTROLES_SITE_MODE=SITE_MODE_TARGET)
    def test_add_peer_on_target(self):
        """Test adding peer site on target site (should fail)"""
        self.options['mode'] = SITE_MODE_PEER
        self.assertEqual(RemoteSite.objects.count(), 0)
        with self.assertRaises(SystemExit) as cm:
            self.command.handle(**self.options)
        self.assertEqual(cm.exception.code, 1)
        self.assertEqual(RemoteSite.objects.count(), 0)

    def test_add_second_target(self):
        """Test adding second target site"""
        self.make_site('Existing Site', 'https://another.site.org')
        self.assertEqual(RemoteSite.objects.count(), 1)
        self.command.handle(**self.options)
        self.assertEqual(RemoteSite.objects.count(), 2)

    def test_add_second_target_existing_name(self):
        """Test adding second target site with existing name (should fail)"""
        self.make_site(REMOTE_SITE_NAME, 'https://another.site.org')
        self.assertEqual(RemoteSite.objects.count(), 1)
        with self.assertRaises(SystemExit) as cm:
            self.command.handle(**self.options)
        self.assertEqual(cm.exception.code, 1)
        self.assertEqual(RemoteSite.objects.count(), 1)

    def test_add_second_target_existing_url(self):
        """Test adding second target site with existing URL (should fail)"""
        self.make_site('Existing Site', REMOTE_SITE_URL)
        self.assertEqual(RemoteSite.objects.count(), 1)
        with self.assertRaises(SystemExit) as cm:
            self.command.handle(**self.options)
        self.assertEqual(cm.exception.code, 1)
        self.assertEqual(RemoteSite.objects.count(), 1)

    def test_add_second_target_existing_url_suppress(self):
        """Test adding second target site with suppressed error"""
        self.options['suppress_error'] = True
        self.make_site('Existing Site', REMOTE_SITE_URL)
        self.assertEqual(RemoteSite.objects.count(), 1)
        with self.assertRaises(SystemExit) as cm:
            self.command.handle(**self.options)
        self.assertEqual(cm.exception.code, 0)  # No error
        self.assertEqual(RemoteSite.objects.count(), 1)

    @override_settings(PROJECTROLES_SITE_MODE=SITE_MODE_TARGET)
    def test_add_second_source(self):
        """Test adding second source site (should fail)"""
        self.make_site(
            'Existing Site', 'https://another.site.org', mode=SITE_MODE_SOURCE
        )
        self.assertEqual(RemoteSite.objects.count(), 1)
        with self.assertRaises(SystemExit) as cm:
            self.command.handle(**self.options)
        self.assertEqual(cm.exception.code, 1)
        self.assertEqual(RemoteSite.objects.count(), 1)


class TestBatchUpdateRoles(
    ProjectMixin,
    RoleMixin,
    RoleAssignmentMixin,
    ProjectInviteMixin,
    BatchUpdateRolesMixin,
    TestCase,
):
    """Tests for batchupdateroles command"""

    def setUp(self):
        super().setUp()
        # Init roles
        self.init_roles()
        # Init users
        self.user_owner = self.make_user('owner')
        self.user_owner.email = 'owner_user@example.com'
        self.user_owner.save()
        self.user_owner_cat = self.make_user('owner_cat')
        self.user_owner_cat.email = 'cat_owner_user@example.com'
        self.user_owner_cat.save()

        # Init projects
        self.category = self.make_project(
            'top_category', PROJECT_TYPE_CATEGORY, None
        )
        self.cat_owner_as = self.make_assignment(
            self.category, self.user_owner_cat, self.role_owner
        )
        self.project = self.make_project(
            'sub_project', PROJECT_TYPE_PROJECT, self.category
        )
        self.owner_as = self.make_assignment(
            self.project, self.user_owner, self.role_owner
        )
        # Save UUIDs for easy access
        self.category_uuid = str(self.category.sodar_uuid)
        self.project_uuid = str(self.project.sodar_uuid)

        # Init command class
        self.command = BatchUpdateRolesCommand()
        # Init file
        self.file = NamedTemporaryFile(delete=False)

    def tearDown(self):
        if self.file:
            os.remove(self.file.name)
        super().tearDown()

    def test_invite(self):
        """Test inviting a single user via email to project"""
        email = 'new@example.com'
        self.assertEqual(ProjectInvite.objects.count(), 0)

        self.write_file([self.project_uuid, email, PROJECT_ROLE_GUEST])
        self.command.handle(
            **{'file': self.file.name, 'issuer': self.user_owner.username}
        )

        self.assertEqual(ProjectInvite.objects.count(), 1)
        invite = ProjectInvite.objects.first()
        self.assertEqual(invite.email, email)
        self.assertEqual(invite.project, self.project)
        self.assertEqual(invite.role, self.role_guest)
        self.assertEqual(invite.issuer, self.user_owner)
        self.assertEqual(len(mail.outbox), 1)

    def test_invite_existing(self):
        """Test inviting a user when they already have an active invite"""
        email = 'new@example.com'
        self.make_invite(email, self.project, self.role_guest, self.user_owner)
        self.assertEqual(ProjectInvite.objects.count(), 1)

        self.write_file([self.project_uuid, email, PROJECT_ROLE_GUEST])
        self.command.handle(
            **{'file': self.file.name, 'issuer': self.user_owner.username}
        )

        self.assertEqual(ProjectInvite.objects.count(), 1)
        self.assertEqual(len(mail.outbox), 0)

    def test_invite_multi_user(self):
        """Test inviting multiple users"""
        email = 'new@example.com'
        email2 = 'new2@example.com'
        self.assertEqual(ProjectInvite.objects.count(), 0)

        fd = [
            [self.project_uuid, email, PROJECT_ROLE_GUEST],
            [self.project_uuid, email2, PROJECT_ROLE_GUEST],
        ]
        self.write_file(fd)
        self.command.handle(
            **{'file': self.file.name, 'issuer': self.user_owner.username}
        )

        self.assertEqual(ProjectInvite.objects.count(), 2)
        self.assertEqual(len(mail.outbox), 2)

    def test_invite_multi_project(self):
        """Test inviting user to multiple projects"""
        email = 'new@example.com'
        self.assertEqual(ProjectInvite.objects.count(), 0)

        fd = [
            [self.category_uuid, email, PROJECT_ROLE_GUEST],
            [self.project_uuid, email, PROJECT_ROLE_GUEST],
        ]
        self.write_file(fd)
        # NOTE: Using user_owner_cat as they have perms for both projects
        self.command.handle(
            **{'file': self.file.name, 'issuer': self.user_owner_cat.username}
        )

        self.assertEqual(
            ProjectInvite.objects.filter(project=self.category).count(), 1
        )
        self.assertEqual(
            ProjectInvite.objects.filter(project=self.project).count(), 1
        )
        self.assertEqual(len(mail.outbox), 2)

    def test_invite_owner(self):
        """Test inviting an owner (should fail)"""
        self.write_file(
            [self.project_uuid, 'new@example.com', PROJECT_ROLE_OWNER]
        )
        self.command.handle(
            **{'file': self.file.name, 'issuer': self.user_owner.username}
        )
        self.assertEqual(ProjectInvite.objects.count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    def test_invite_delegate(self):
        """Test inviting a delegate"""
        self.write_file(
            [self.project_uuid, 'new@example.com', PROJECT_ROLE_DELEGATE]
        )
        self.command.handle(
            **{'file': self.file.name, 'issuer': self.user_owner.username}
        )
        self.assertEqual(ProjectInvite.objects.count(), 1)
        self.assertEqual(len(mail.outbox), 1)

    @override_settings(PROJECTROLES_DELEGATE_LIMIT=2)
    def test_invite_delegate_no_perms(self):
        """Test inviting a delegate without perms (should fail)"""
        user_delegate = self.make_user('delegate')
        self.make_assignment(self.project, user_delegate, self.role_delegate)
        self.write_file(
            [self.project_uuid, 'new@example.com', PROJECT_ROLE_DELEGATE]
        )
        self.command.handle(
            **{'file': self.file.name, 'issuer': user_delegate.username}
        )
        self.assertEqual(ProjectInvite.objects.count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    def test_invite_delegate_limit(self):
        """Test inviting a delegate with limit reached (should fail)"""
        user_delegate = self.make_user('delegate')
        self.make_assignment(self.project, user_delegate, self.role_delegate)
        self.write_file(
            [self.project_uuid, 'new@example.com', PROJECT_ROLE_DELEGATE]
        )
        self.command.handle(
            **{'file': self.file.name, 'issuer': self.user_owner.username}
        )
        self.assertEqual(ProjectInvite.objects.count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    def test_role_add(self):
        """Test adding role to user already in system"""
        email = 'new@example.com'
        user_new = self.make_user('user_new')
        user_new.email = email
        user_new.save()
        self.assertEqual(ProjectInvite.objects.count(), 0)

        self.write_file([self.project_uuid, email, PROJECT_ROLE_GUEST])
        self.command.handle(
            **{'file': self.file.name, 'issuer': self.user_owner.username}
        )

        self.assertEqual(ProjectInvite.objects.count(), 0)
        role_as = RoleAssignment.objects.get(
            project=self.project, user=user_new
        )
        self.assertEqual(role_as.role, self.role_guest)
        self.assertEqual(len(mail.outbox), 1)

    def test_role_update(self):
        """Test updating an existing role for user"""
        email = 'new@example.com'
        user_new = self.make_user('user_new')
        user_new.email = email
        user_new.save()
        role_as = self.make_assignment(self.project, user_new, self.role_guest)
        self.assertEqual(
            RoleAssignment.objects.filter(
                project=self.project, user=user_new
            ).count(),
            1,
        )

        self.write_file([self.project_uuid, email, PROJECT_ROLE_CONTRIBUTOR])
        self.command.handle(
            **{'file': self.file.name, 'issuer': self.user_owner.username}
        )

        self.assertEqual(ProjectInvite.objects.count(), 0)
        role_as.refresh_from_db()
        self.assertEqual(role_as.role, self.role_contributor)
        self.assertEqual(len(mail.outbox), 1)

    def test_role_update_owner(self):
        """Test updating the role of current owner (should fail)"""
        email = 'owner@example.com'
        self.user_owner.email = email
        self.user_owner.save()

        self.write_file([self.project_uuid, email, PROJECT_ROLE_CONTRIBUTOR])
        self.command.handle(
            **{'file': self.file.name, 'issuer': self.user_owner.username}
        )

        self.assertEqual(ProjectInvite.objects.count(), 0)
        self.owner_as.refresh_from_db()
        self.assertEqual(self.owner_as.role, self.role_owner)
        self.assertEqual(
            RoleAssignment.objects.filter(
                project=self.project, user=self.user_owner
            ).count(),
            1,
        )
        self.assertEqual(len(mail.outbox), 0)

    @override_settings(PROJECTROLES_DELEGATE_LIMIT=2)
    def test_role_update_delegate_no_perms(self):
        """Test updating a delegate role without perms (should fail)"""
        email = 'new@example.com'
        user_new = self.make_user('user_new')
        user_new.email = email
        user_new.save()
        role_as = self.make_assignment(self.project, user_new, self.role_guest)
        user_delegate = self.make_user('delegate')
        self.make_assignment(self.project, user_delegate, self.role_delegate)

        self.write_file([self.project_uuid, email, PROJECT_ROLE_DELEGATE])
        self.command.handle(
            **{'file': self.file.name, 'issuer': user_delegate.username}
        )

        self.assertEqual(ProjectInvite.objects.count(), 0)
        role_as.refresh_from_db()
        self.assertEqual(role_as.role, self.role_guest)
        self.assertEqual(len(mail.outbox), 0)

    def test_role_update_delegate_limit(self):
        """Test updating a delegate role with limit reached (should fail)"""
        email = 'new@example.com'
        user_new = self.make_user('user_new')
        user_new.email = email
        user_new.save()
        role_as = self.make_assignment(self.project, user_new, self.role_guest)
        user_delegate = self.make_user('delegate')
        self.make_assignment(self.project, user_delegate, self.role_delegate)

        self.write_file([self.project_uuid, email, PROJECT_ROLE_DELEGATE])
        self.command.handle(
            **{'file': self.file.name, 'issuer': self.user_owner.username}
        )

        self.assertEqual(ProjectInvite.objects.count(), 0)
        role_as.refresh_from_db()
        self.assertEqual(role_as.role, self.role_guest)
        self.assertEqual(len(mail.outbox), 0)

    def test_role_add_inherited_owner(self):
        """Test adding role with inherited owner rank (should fail)"""
        self.write_file(
            [
                self.project_uuid,
                self.user_owner_cat.email,
                PROJECT_ROLE_CONTRIBUTOR,
            ]
        )
        self.command.handle(
            **{'file': self.file.name, 'issuer': self.user_owner.username}
        )

        self.assertEqual(ProjectInvite.objects.count(), 0)
        self.assertEqual(
            RoleAssignment.objects.filter(
                project=self.project, user=self.user_owner_cat
            ).count(),
            0,
        )
        self.assertEqual(len(mail.outbox), 0)

    def test_role_add_inherited_higher(self):
        """Test adding role with inherited role of higher rank"""
        user_new = self.make_user('user_new')
        user_new.email = 'user_new@example.com'
        self.make_assignment(self.category, user_new, self.role_guest)

        self.write_file(
            [
                self.project_uuid,
                user_new.email,
                PROJECT_ROLE_CONTRIBUTOR,
            ]
        )
        self.command.handle(
            **{'file': self.file.name, 'issuer': self.user_owner.username}
        )

        self.assertEqual(ProjectInvite.objects.count(), 0)
        self.assertEqual(
            RoleAssignment.objects.filter(
                project=self.project, user=user_new
            ).count(),
            0,
        )
        self.assertEqual(len(mail.outbox), 0)

    def test_role_add_inherited_equal(self):
        """Test adding role with inherited role of equal rank (should fail)"""
        user_new = self.make_user('user_new')
        user_new.email = 'user_new@example.com'
        self.make_assignment(self.category, user_new, self.role_contributor)

        self.write_file(
            [
                self.project_uuid,
                user_new.email,
                PROJECT_ROLE_CONTRIBUTOR,
            ]
        )
        self.command.handle(
            **{'file': self.file.name, 'issuer': self.user_owner.username}
        )

        self.assertEqual(ProjectInvite.objects.count(), 0)
        self.assertEqual(
            RoleAssignment.objects.filter(
                project=self.project, user=user_new
            ).count(),
            0,
        )
        self.assertEqual(len(mail.outbox), 0)

    def test_role_add_inherited_lower(self):
        """Test adding role with inherited role of lower rank (should fail)"""
        user_new = self.make_user('user_new')
        user_new.email = 'user_new@example.com'
        self.make_assignment(self.category, user_new, self.role_contributor)

        self.write_file([self.project_uuid, user_new.email, PROJECT_ROLE_GUEST])
        self.command.handle(
            **{'file': self.file.name, 'issuer': self.user_owner.username}
        )

        self.assertEqual(ProjectInvite.objects.count(), 0)
        self.assertEqual(
            RoleAssignment.objects.filter(
                project=self.project, user=user_new
            ).count(),
            0,
        )
        self.assertEqual(len(mail.outbox), 0)

    def test_invite_and_update(self):
        """Test inviting and updating in one command"""
        email = 'new@example.com'
        user_new = self.make_user('user_new')
        user_new.email = email
        user_new.save()
        email2 = 'new2@example.com'
        self.assertEqual(ProjectInvite.objects.count(), 0)

        fd = [
            [self.project_uuid, email, PROJECT_ROLE_GUEST],
            [self.project_uuid, email2, PROJECT_ROLE_GUEST],
        ]
        self.write_file(fd)
        self.command.handle(
            **{'file': self.file.name, 'issuer': self.user_owner.username}
        )

        self.assertEqual(ProjectInvite.objects.count(), 1)
        self.assertIsNotNone(
            RoleAssignment.objects.filter(
                project=self.project, user=user_new
            ).count(),
            1,
        )
        self.assertEqual(len(mail.outbox), 2)

    def test_command_no_issuer(self):
        """Test invite without issuer (should go under admin)"""
        admin = self.make_user(settings.PROJECTROLES_DEFAULT_ADMIN)
        admin.is_superuser = True
        admin.save()
        email = 'new@example.com'
        self.write_file([self.project_uuid, email, PROJECT_ROLE_GUEST])
        self.command.handle(**{'file': self.file.name, 'issuer': None})
        invite = ProjectInvite.objects.first()
        self.assertEqual(invite.issuer, admin)

    def test_command_no_perms(self):
        """Test invite with issuer who lacks permissions for a project"""
        issuer = self.make_user('issuer')
        email = 'new@example.com'
        self.write_file([self.project_uuid, email, PROJECT_ROLE_GUEST])
        self.command.handle(
            **{'file': self.file.name, 'issuer': issuer.username}
        )
        self.assertEqual(ProjectInvite.objects.count(), 0)
        self.assertEqual(len(mail.outbox), 0)


class TestCleanAppSettings(
    ProjectMixin, RoleMixin, RoleAssignmentMixin, AppSettingMixin, TestCase
):
    """Tests for cleanappsettings command and associated functions"""

    def setUp(self):
        super().setUp()
        # Init roles
        self.init_roles()
        # Init users
        self.user_owner = self.make_user('owner')
        self.user_owner.email = 'owner_user@example.com'
        self.user_owner.save()

        # Init projects
        self.category = self.make_project(
            'top_category', PROJECT_TYPE_CATEGORY, None
        )
        self.cat_owner_as = self.make_assignment(
            self.category, self.user_owner, self.role_owner
        )
        self.project = self.make_project(
            'sub_project', PROJECT_TYPE_PROJECT, self.category
        )
        self.owner_as = self.make_assignment(
            self.project, self.user_owner, self.role_owner
        )
        self.plugin = Plugin.objects.get(name='example_project_app')

        # Init test setting
        self.setting_str_values = {
            'plugin_name': EXAMPLE_APP_NAME,
            'project': self.project,
            'name': 'project_str_setting',
            'setting_type': 'STRING',
            'value': 'test',
            'update_value': 'better test',
            'non_valid_value': False,
        }
        self.setting_int_values = {
            'plugin_name': EXAMPLE_APP_NAME,
            'project': self.project,
            'name': 'project_int_setting',
            'setting_type': 'INTEGER',
            'value': 0,
            'update_value': 170,
            'non_valid_value': 'Nan',
        }
        self.setting_bool_values = {
            'plugin_name': EXAMPLE_APP_NAME,
            'project': self.project,
            'name': 'project_bool_setting',
            'setting_type': 'BOOLEAN',
            'value': False,
            'update_value': True,
            'non_valid_value': 170,
        }
        self.setting_json_values = {
            'plugin_name': EXAMPLE_APP_NAME,
            'project': self.project,
            'name': 'project_json_setting',
            'setting_type': 'JSON',
            'value': {
                'Example': 'Value',
                'list': [1, 2, 3, 4, 5],
                'level_6': False,
            },
            'update_value': {'Test_more': 'often_always'},
            'non_valid_value': self.project,
        }
        self.setting_iprestrict_values = {
            'plugin_name': 'projectroles',
            'project': self.project,
            'name': 'ip_restrict',
            'setting_type': 'BOOLEAN',
            'value': False,
            'update_value': True,
            'non_valid_value': 170,
        }
        self.settings = [
            self.setting_int_values,
            self.setting_json_values,
            self.setting_str_values,
            self.setting_bool_values,
            self.setting_iprestrict_values,
        ]
        for s in self.settings:
            self.make_setting(
                plugin_name=s['plugin_name'],
                name=s['name'],
                setting_type=s['setting_type'],
                value=s['value'] if s['setting_type'] != 'JSON' else '',
                value_json=s['value'] if s['setting_type'] == 'JSON' else {},
                project=s['project'],
            )

    def test_command_undefined(self):
        """Test cleanappsettings with undefined setting"""
        undef_setting = AppSetting(
            app_plugin=self.plugin,
            project=self.project,
            name='ghost',
            type='BOOLEAN',
            value=True,
        )
        undef_setting.save()
        self.assertEqual(AppSetting.objects.count(), 6)

        with self.assertLogs(
            'projectroles.management.commands.cleanappsettings', level='INFO'
        ) as cm:
            call_command('cleanappsettings')
            self.assertEqual(
                cm.output,
                [
                    CLEAN_LOG_PREFIX + START_MSG,
                    (
                        CLEAN_LOG_PREFIX
                        + DELETE_PREFIX_MSG.format(
                            'settings.example_project_app.ghost',
                            self.project.title,
                        )
                        + DEFINITION_NOT_FOUND_MSG
                    ),
                    CLEAN_LOG_PREFIX + END_MSG,
                ],
            )
        self.assertEqual(AppSetting.objects.count(), 5)
        self.assertQuerysetEqual(
            AppSetting.objects.filter(name='ghost'),
            [],
        )

    def test_command_project_types(self):
        """Test cleanappsettings with invalid project types"""
        cat_setting = AppSetting(
            app_plugin=self.plugin,
            project=self.category,
            name='project_user_bool_setting',
            type='BOOLEAN',
            value=True,
        )
        cat_setting.save()
        self.assertEqual(AppSetting.objects.count(), 6)

        with self.assertLogs(
            'projectroles.management.commands.cleanappsettings', level='INFO'
        ) as cm:
            call_command('cleanappsettings')
            self.assertEqual(
                cm.output,
                [
                    CLEAN_LOG_PREFIX + START_MSG,
                    (
                        CLEAN_LOG_PREFIX
                        + DELETE_PREFIX_MSG.format(
                            'settings.example_project_app.'
                            'project_user_bool_setting',
                            self.category.title,
                        )
                        + DELETE_PROJECT_TYPE_MSG.format(
                            self.category.type,
                            ALLOWED_TYPES_MSG,
                            '[\'PROJECT\']',
                        )
                    ),
                    CLEAN_LOG_PREFIX + END_MSG,
                ],
            )
            self.assertEqual(AppSetting.objects.count(), 5)
            self.assertQuerysetEqual(
                AppSetting.objects.filter(name='project_user_bool_setting'),
                [],
            )

    def test_command_project_user_scope(self):
        """Test cleanappsettings with PROJECT_USER scope"""
        user_new = self.make_user('user_new')
        user_new.save()
        pu_setting = AppSetting(
            app_plugin=self.plugin,
            project=self.project,
            user=user_new,
            name='project_user_bool_setting',
            type='BOOLEAN',
            value=True,
        )
        pu_setting.save()
        self.assertEqual(AppSetting.objects.count(), 6)

        with self.assertLogs(
            'projectroles.management.commands.cleanappsettings', level='INFO'
        ) as cm:
            call_command('cleanappsettings')
            self.assertEqual(
                cm.output,
                [
                    CLEAN_LOG_PREFIX + START_MSG,
                    (
                        CLEAN_LOG_PREFIX
                        + DELETE_PREFIX_MSG.format(
                            'settings.example_project_app.'
                            'project_user_bool_setting',
                            self.project.title,
                        )
                        + DELETE_SCOPE_MSG.format(
                            user_new.username,
                        )
                    ),
                    CLEAN_LOG_PREFIX + END_MSG,
                ],
            )
            self.assertEqual(AppSetting.objects.count(), 5)
            self.assertQuerysetEqual(
                AppSetting.objects.filter(name='project_user_bool_setting'),
                [],
            )


class TestSyncGroups(TestCase):
    """Tests for syncgroups command"""

    def setUp(self):
        self.command = SyncGroupsCommand()

    def test_sync_system(self):
        """Test sync with system username"""
        user = self.make_user('user')
        user.groups.all().delete()  # Remove groups
        self.assertEqual(user.groups.count(), 0)
        self.command.handle()
        self.assertEqual(user.groups.count(), 1)
        group = user.groups.first()
        self.assertIsNotNone(group)
        self.assertEqual(group.name, SYSTEM_USER_GROUP)

    def test_sync_system_existing(self):
        """Test sync with system username and existing group"""
        user = self.make_user('user')
        self.assertEqual(user.groups.count(), 1)
        self.command.handle()
        self.assertEqual(user.groups.count(), 1)
        group = user.groups.first()
        self.assertIsNotNone(group)
        self.assertEqual(group.name, SYSTEM_USER_GROUP)

    @override_settings(AUTH_LDAP_USERNAME_DOMAIN=LDAP_DOMAIN)
    def test_sync_ldap(self):
        """Test sync with LDAP user"""
        user = self.make_user('user@' + LDAP_DOMAIN)
        user.groups.all().delete()
        self.assertEqual(user.groups.count(), 0)
        self.command.handle()
        self.assertEqual(user.groups.count(), 1)
        group = user.groups.first()
        self.assertIsNotNone(group)
        self.assertEqual(group.name, LDAP_DOMAIN.lower())

    @override_settings(AUTH_LDAP_USERNAME_DOMAIN=LDAP_DOMAIN)
    def test_sync_ldap_existing(self):
        """Test sync with LDAP user and existing group"""
        user = self.make_user('user@' + LDAP_DOMAIN)
        self.assertEqual(user.groups.count(), 1)
        self.command.handle()
        self.assertEqual(user.groups.count(), 1)
        group = user.groups.first()
        self.assertIsNotNone(group)
        self.assertEqual(group.name, LDAP_DOMAIN.lower())


@override_settings(DEBUG=True)
class TestCreateDevUsers(TestCase):
    """Tests for createdevusers command"""

    def test_create(self):
        """Test creating users"""
        self.assertEqual(User.objects.count(), 0)
        call_command('createdevusers')
        self.assertEqual(User.objects.count(), len(DEV_USER_NAMES))
        for u in DEV_USER_NAMES:
            user = User.objects.filter(
                username=u,
                name='{} Example'.format(u.capitalize()),
                first_name=u.capitalize(),
                last_name='Example',
                email='{}@example.com'.format(u),
            ).first()
            self.assertIsNotNone(user)
            self.assertEqual(user.check_password(DEFAULT_PASSWORD), True)

    def test_create_custom_password(self):
        """Test creating users with custom password"""
        call_command('createdevusers', password=CUSTOM_PASSWORD)
        for u in DEV_USER_NAMES:
            user = User.objects.filter(username=u).first()
            self.assertEqual(user.check_password(CUSTOM_PASSWORD), True)

    def test_create_existing(self):
        """Test creating users with existing user in list"""
        self.make_user(DEV_USER_NAMES[0])
        self.assertEqual(User.objects.count(), 1)
        call_command('createdevusers')
        self.assertEqual(User.objects.count(), len(DEV_USER_NAMES))

    def test_create_existing_not_in_list(self):
        """Test creating users with existing user not in list"""
        self.make_user('user_new')
        self.assertEqual(User.objects.count(), 1)
        call_command('createdevusers')
        self.assertEqual(User.objects.count(), len(DEV_USER_NAMES) + 1)

    @override_settings(DEBUG=False)
    def test_create_no_debug(self):
        """Test creating users with debug mode disabled (should fail)"""
        self.assertEqual(User.objects.count(), 0)
        with self.assertRaises(SystemExit):
            call_command('createdevusers')
        self.assertEqual(User.objects.count(), 0)



"""UI tests for the tokens app"""

from datetime import timedelta

from django.urls import reverse
from django.utils import timezone

from knox.models import AuthToken
from selenium.webdriver.common.by import By

# Projectroles dependency
from projectroles.tests.test_ui import UITestBase


class TestTokenList(UITestBase):
    """Tests for token list"""

    def setUp(self):
        super().setUp()
        # Create users
        self.superuser = self.make_user('superuser', True)
        self.superuser.is_superuser = True
        self.superuser.is_staff = True
        self.superuser.save()
        self.regular_user = self.make_user('regular_user', False)

        # Create tokens
        self.token = AuthToken.objects.create(
            self.regular_user, timedelta(days=5)
        )[0]
        self.token_no_expiry = AuthToken.objects.create(
            self.regular_user, None
        )[0]
        self.url = reverse('tokens:list')

    def test_list_items(self):
        """Test visibility of items in token list"""
        expected = [(self.superuser, 0), (self.regular_user, 2)]
        self.assert_element_count(
            expected, self.url, 'sodar-tk-list-item', 'class'
        )

    def test_list_expiry(self):
        """Test token expiry dates in token list"""
        self.login_and_redirect(self.regular_user, self.url)
        items = self.selenium.find_elements(By.CLASS_NAME, 'sodar-tk-list-item')
        self.assertEqual(len(items), 2)
        expiry_time = timezone.localtime(self.token.expiry).strftime(
            '%Y-%m-%d %H:%M'
        )
        values = []
        for item in items:
            values.append(item.find_elements(By.TAG_NAME, 'td')[2].text)
        self.assertCountEqual(values, ['Never', expiry_time])



# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-07-05 13:05
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0010_update_appsetting'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='roleassignment',
            name='projectrole_project_b13795_idx',
        ),
        migrations.RemoveIndex(
            model_name='roleassignment',
            name='projectrole_user_id_0e46f7_idx',
        ),
    ]



import json

from django import forms
from django.conf import settings

# Projectroles dependency
from projectroles.app_settings import AppSettingAPI
from projectroles.forms import (
    SODARForm,
    SODARModelForm,
    SETTING_CUSTOM_VALIDATE_MSG,
    SETTING_DISABLE_LABEL,
    SETTING_SOURCE_ONLY_MSG,
)
from projectroles.models import (
    SODARUserAdditionalEmail,
    SODAR_CONSTANTS,
    ADD_EMAIL_ALREADY_SET_MSG,
)
from projectroles.plugins import get_active_plugins
from projectroles.utils import build_secret


app_settings = AppSettingAPI()


# SODAR Constants
SITE_MODE_SOURCE = SODAR_CONSTANTS['SITE_MODE_SOURCE']
SITE_MODE_TARGET = SODAR_CONSTANTS['SITE_MODE_TARGET']
APP_SETTING_SCOPE_USER = SODAR_CONSTANTS['APP_SETTING_SCOPE_USER']


class UserSettingsForm(SODARForm):
    """Form for configuring user settings"""

    def _set_app_setting_field(self, plugin_name, s_field, s_key, s_val):
        """
        Set user app setting field, widget and value.

        :param plugin_name: App plugin name
        :param s_field: Form field name
        :param s_key: Setting key
        :param s_val: Setting value
        """
        s_widget_attrs = s_val.get('widget_attrs') or {}
        if 'placeholder' in s_val:
            s_widget_attrs['placeholder'] = s_val.get('placeholder')
        setting_kwargs = {
            'required': False,
            'label': s_val.get('label') or '{}.{}'.format(plugin_name, s_key),
            'help_text': s_val.get('description'),
        }
        # Disable global user settings if on target site
        if (
            app_settings.get_global_value(s_val)
            and settings.PROJECTROLES_SITE_MODE == SITE_MODE_TARGET
        ):
            setting_kwargs['label'] += ' ' + SETTING_DISABLE_LABEL
            setting_kwargs['help_text'] += ' ' + SETTING_SOURCE_ONLY_MSG
            setting_kwargs['disabled'] = True

        if s_val.get('options') and callable(s_val['options']):
            self.fields[s_field] = forms.ChoiceField(
                choices=[
                    (
                        (str(value[0]), str(value[1]))
                        if isinstance(value, tuple)
                        else (str(value), str(value))
                    )
                    for value in s_val['options'](user=self.user)
                ],
                **setting_kwargs,
            )
        elif s_val.get('options'):
            self.fields[s_field] = forms.ChoiceField(
                choices=[
                    (
                        (int(option), int(option))
                        if s_val['type'] == 'INTEGER'
                        else (option, option)
                    )
                    for option in s_val['options']
                ],
                **setting_kwargs,
            )
        elif s_val['type'] == 'STRING':
            self.fields[s_field] = forms.CharField(
                widget=forms.TextInput(attrs=s_widget_attrs),
                **setting_kwargs,
            )
        elif s_val['type'] == 'INTEGER':
            self.fields[s_field] = forms.IntegerField(
                widget=forms.NumberInput(attrs=s_widget_attrs),
                **setting_kwargs,
            )
        elif s_val['type'] == 'BOOLEAN':
            self.fields[s_field] = forms.BooleanField(**setting_kwargs)
        elif s_val['type'] == 'JSON':
            # NOTE: Attrs MUST be supplied here (#404)
            if 'class' in s_widget_attrs:
                s_widget_attrs['class'] += ' sodar-json-input'
            else:
                s_widget_attrs['class'] = 'sodar-json-input'
            self.fields[s_field] = forms.CharField(
                widget=forms.Textarea(attrs=s_widget_attrs),
                **setting_kwargs,
            )

        # Modify initial value and attributes
        self.fields[s_field].widget.attrs.update(s_widget_attrs)
        value = app_settings.get(
            plugin_name=plugin_name, setting_name=s_key, user=self.user
        )
        if s_val['type'] == 'JSON':
            value = json.dumps(value)
        self.initial[s_field] = value

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('current_user')
        super().__init__(*args, **kwargs)
        # Add settings fields
        self.app_plugins = get_active_plugins(plugin_type='project_app')
        self.user_plugins = get_active_plugins(plugin_type='site_app')
        self.app_plugins = self.app_plugins + self.user_plugins

        for plugin in self.app_plugins + [None]:
            if plugin:
                name = plugin.name
                p_defs = app_settings.get_definitions(
                    APP_SETTING_SCOPE_USER, plugin=plugin, user_modifiable=True
                )
            else:
                name = 'projectroles'
                p_defs = app_settings.get_definitions(
                    APP_SETTING_SCOPE_USER,
                    plugin_name=name,
                    user_modifiable=True,
                )
            for s_key, s_val in p_defs.items():
                s_field = 'settings.{}.{}'.format(name, s_key)
                self._set_app_setting_field(name, s_field, s_key, s_val)
                self.fields[s_field].label = self.get_app_setting_label(
                    plugin, self.fields[s_field].label
                )

    def clean(self):
        """Function for custom form validation and cleanup"""
        for plugin in self.app_plugins + [None]:
            p_kwargs = {'user_modifiable': True}
            if plugin:
                p_name = plugin.name
                p_kwargs['plugin'] = plugin
            else:
                p_name = 'projectroles'
                p_kwargs['plugin_name'] = p_name
            p_defs = app_settings.get_definitions(
                APP_SETTING_SCOPE_USER, **p_kwargs
            )
            p_settings = {}

            for s_key, s_val in p_defs.items():
                s_field = '.'.join(['settings', p_name, s_key])
                p_settings[s_key] = self.cleaned_data.get(s_field)

                if s_val['type'] == 'JSON':
                    if not self.cleaned_data.get(s_field):
                        self.cleaned_data[s_field] = '{}'
                    try:
                        self.cleaned_data[s_field] = json.loads(
                            self.cleaned_data.get(s_field)
                        )
                    except json.JSONDecodeError as err:
                        self.add_error(s_field, 'Invalid JSON\n' + str(err))
                elif s_val['type'] == 'INTEGER':
                    # Convert integers from select fields
                    self.cleaned_data[s_field] = int(self.cleaned_data[s_field])

                if not app_settings.validate(
                    setting_type=s_val['type'],
                    setting_value=self.cleaned_data.get(s_field),
                    setting_options=s_val.get('options'),
                    user=self.user,
                ):
                    self.add_error(s_field, 'Invalid value')

            # Custom plugin validation for app settings
            if plugin and hasattr(plugin, 'validate_form_app_settings'):
                try:
                    p_errors = plugin.validate_form_app_settings(
                        p_settings, user=self.user
                    )
                    if p_errors:
                        for field, error in p_errors.items():
                            f_name = '.'.join(['settings', p_name, field])
                            self.add_error(f_name, error)
                except Exception as ex:
                    self.add_error(
                        None,
                        SETTING_CUSTOM_VALIDATE_MSG.format(
                            plugin=p_name, exception=ex
                        ),
                    )
        return self.cleaned_data


class UserEmailForm(SODARModelForm):
    """Form for creating additional email addresses for user"""

    class Meta:
        model = SODARUserAdditionalEmail
        fields = ['email', 'user', 'secret']

    def __init__(self, current_user=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if current_user:
            self.current_user = current_user
        self.fields['user'].widget = forms.HiddenInput()
        self.initial['user'] = current_user
        self.fields['secret'].widget = forms.HiddenInput()
        self.initial['secret'] = build_secret(32)

    def clean(self):
        if self.cleaned_data['email'] == self.current_user.email:
            self.add_error(
                'email', ADD_EMAIL_ALREADY_SET_MSG.format(email_type='primary')
            )
            return self.cleaned_data
        if (
            SODARUserAdditionalEmail.objects.filter(
                user=self.current_user, email=self.cleaned_data['email']
            ).count()
            > 0
        ):
            self.add_error(
                'email',
                ADD_EMAIL_ALREADY_SET_MSG.format(email_type='additional'),
            )
        return self.cleaned_data



# Generated by Django 4.2.11 on 2024-03-21 12:29

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("projectroles", "0028_populate_finder_role"),
        ("filesfolders", "0004_update_uuid"),
    ]

    operations = [
        migrations.AlterField(
            model_name="file",
            name="folder",
            field=models.ForeignKey(
                blank=True,
                help_text="Folder under which object exists (null if root folder)",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(app_label)s_%(class)s_children",
                to="filesfolders.folder",
            ),
        ),
        migrations.AlterField(
            model_name="file",
            name="project",
            field=models.ForeignKey(
                help_text="Project in which the object belongs",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(app_label)s_%(class)s_objects",
                to="projectroles.project",
            ),
        ),
        migrations.AlterField(
            model_name="folder",
            name="folder",
            field=models.ForeignKey(
                blank=True,
                help_text="Folder under which object exists (null if root folder)",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(app_label)s_%(class)s_children",
                to="filesfolders.folder",
            ),
        ),
        migrations.AlterField(
            model_name="folder",
            name="project",
            field=models.ForeignKey(
                help_text="Project in which the object belongs",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(app_label)s_%(class)s_objects",
                to="projectroles.project",
            ),
        ),
        migrations.AlterField(
            model_name="hyperlink",
            name="folder",
            field=models.ForeignKey(
                blank=True,
                help_text="Folder under which object exists (null if root folder)",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(app_label)s_%(class)s_children",
                to="filesfolders.folder",
            ),
        ),
        migrations.AlterField(
            model_name="hyperlink",
            name="project",
            field=models.ForeignKey(
                help_text="Project in which the object belongs",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(app_label)s_%(class)s_objects",
                to="projectroles.project",
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-04-11 15:58
from __future__ import unicode_literals

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AdminAlert',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('message', models.CharField(help_text='Alert message to be shown for users', max_length=255)),
                ('description', models.TextField(blank=True, help_text='Full description of alert (optional, will be shown on a separate page)', null=True)),
                ('date_created', models.DateTimeField(auto_now_add=True, help_text='Alert creation timestamp')),
                ('date_expire', models.DateTimeField(help_text='Alert expiration timestamp')),
                ('active', models.BooleanField(default=True, help_text='Alert status (for disabling the alert before expiration)')),
                ('omics_uuid', models.UUIDField(default=uuid.uuid4, help_text='Adminalerts Omics UUID', unique=True)),
            ],
        ),
    ]



# Generated by Django 3.2.17 on 2023-02-02 13:13

from django.db import migrations
import uuid


def gen_uuid(apps, schema_editor):
    ProjectEventStatus = apps.get_model('timeline', 'ProjectEventStatus')
    for status in ProjectEventStatus.objects.all():
        status.sodar_uuid = uuid.uuid4()
        status.save(update_fields=['sodar_uuid'])

class Migration(migrations.Migration):

    dependencies = [
        ('timeline', '0009_create_status_uuid'),
    ]

    operations = [
        migrations.RunPython(gen_uuid, reverse_code=migrations.RunPython.noop)
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-04-16 08:18
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sodarcache', '0002_make_user_optional'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='jsoncacheitem',
            index=models.Index(fields=['project', 'app_name', 'name'], name='sodarcache__project_e54b84_idx'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-10-30 06:52
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [("bgjobs", "0002_backgroundjoblogentry")]

    operations = [
        migrations.AddField(
            model_name="backgroundjob",
            name="date_created",
            field=models.DateTimeField(
                auto_now_add=True,
                default=django.utils.timezone.now,
                help_text="DateTime of creation",
            ),
            preserve_default=False,
        )
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-04-15 11:50
from __future__ import unicode_literals

from django.db import migrations


def fix_project_keys(apps, schema_editor):
    """ Fix missing RemoteProject.project foreign keys (issue #197)"""
    RemoteProject = apps.get_model('projectroles', 'RemoteProject')
    Project = apps.get_model('projectroles', 'Project')

    for rp in RemoteProject.objects.all():
        if not rp.project:
            rp.project = Project.objects.filter(
                sodar_uuid=rp.project_uuid
            ).first()
            rp.save()


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0006_add_remote_projects'),
    ]

    operations = [
        migrations.RunPython(fix_project_keys),
    ]



# Generated by Django 4.2.11 on 2024-04-17 15:55

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("projectroles", "0028_populate_finder_role"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("timeline", "0011_make_uuid_unique"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="ProjectEvent",
            new_name="TimelineEvent",
        ),
        migrations.RenameModel(
            old_name="ProjectEventObjectRef",
            new_name="TimelineEventObjectRef",
        ),
        migrations.RenameModel(
            old_name="ProjectEventStatus",
            new_name="TimelineEventStatus",
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-01-15 12:55
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0013_update_appsetting_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='appsetting',
            name='value_json',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=dict, help_text='Optional JSON value for the setting', null=True),
        ),
    ]



# Generated by Django 3.1.7 on 2021-02-25 14:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('timeline', '0004_update_uuid'),
    ]

    operations = [
        migrations.AlterField(
            model_name='projectevent',
            name='extra_data',
            field=models.JSONField(default=dict, help_text='Additional event data as JSON'),
        ),
        migrations.AlterField(
            model_name='projecteventobjectref',
            name='extra_data',
            field=models.JSONField(default=dict, help_text='Additional data related to the object as JSON'),
        ),
        migrations.AlterField(
            model_name='projecteventstatus',
            name='extra_data',
            field=models.JSONField(default=dict, help_text='Additional status data as JSON'),
        ),
    ]




# This file was generated by 'versioneer.py' (0.29) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "date": "2024-08-08T16:34:20+0200",
 "dirty": false,
 "error": null,
 "full-revisionid": "42d2732f930ca135d76c63f6bfc8c467a0a4f0c2",
 "version": "1.0.1"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)



# Generated by Django 4.2.11 on 2024-04-18 08:18

import uuid

from django.db import migrations

def gen_uuid(apps, schema_editor):
    TimelineEventObjectRef = apps.get_model('timeline', 'TimelineEventObjectRef')
    for status in TimelineEventObjectRef.objects.all():
        status.sodar_uuid = uuid.uuid4()
        status.save(update_fields=['sodar_uuid'])

class Migration(migrations.Migration):

    dependencies = [
        ("timeline", "0013_timelineeventobjectref_sodar_uuid"),
    ]

    operations = [
        migrations.RunPython(gen_uuid, reverse_code=migrations.RunPython.noop),
    ]



# Generated by Django 3.1.7 on 2021-03-16 15:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0018_update_jsonfield'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='public_guest_access',
            field=models.BooleanField(default=False, help_text='Allow public guest access for the project, also including unauthenticated users if allowed on the site'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-04-11 15:58
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import markupfield.fields
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Project',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='Project title', max_length=255)),
                ('type', models.CharField(choices=[('CATEGORY', 'Category'), ('PROJECT', 'Project')], default='PROJECT', help_text='Type of project ("CATEGORY", "PROJECT")', max_length=64)),
                ('description', models.CharField(help_text='Short project description', max_length=512)),
                ('readme', markupfield.fields.MarkupField(blank=True, help_text='Project README (optional, supports markdown)', null=True, rendered_field=True)),
                ('readme_markup_type', models.CharField(choices=[('', '--'), ('html', 'HTML'), ('plain', 'Plain'), ('markdown', 'Markdown'), ('restructuredtext', 'Restructured Text')], default='markdown', editable=False, max_length=30)),
                ('submit_status', models.CharField(default='OK', help_text='Status of project creation', max_length=64)),
                ('_readme_rendered', models.TextField(editable=False, null=True)),
                ('omics_uuid', models.UUIDField(default=uuid.uuid4, help_text='Project Omics UUID', unique=True)),
            ],
            options={
                'ordering': ['parent__title', 'title'],
            },
        ),
        migrations.CreateModel(
            name='ProjectInvite',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(help_text='Email address of the person to be invited', max_length=254)),
                ('date_created', models.DateTimeField(auto_now_add=True, help_text='DateTime of invite creation')),
                ('date_expire', models.DateTimeField(help_text='Expiration of invite as DateTime')),
                ('message', models.TextField(blank=True, help_text='Message to be included in the invite email (optional)')),
                ('secret', models.CharField(help_text='Secret token provided to user with the invite', max_length=255, unique=True)),
                ('active', models.BooleanField(default=True, help_text='Status of the invite (False if claimed or revoked)')),
                ('omics_uuid', models.UUIDField(default=uuid.uuid4, help_text='ProjectInvite Omics UUID', unique=True)),
            ],
            options={
                'ordering': ['project__title', 'email', 'role__name'],
            },
        ),
        migrations.CreateModel(
            name='ProjectSetting',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Name of the setting', max_length=255)),
                ('type', models.CharField(choices=[('BOOLEAN', 'Boolean'), ('INTEGER', 'Integer'), ('STRING', 'String')], help_text='Type of the setting', max_length=64)),
                ('value', models.CharField(blank=True, help_text='Value of the setting', max_length=255, null=True)),
                ('omics_uuid', models.UUIDField(default=uuid.uuid4, help_text='ProjectSetting Omics UUID', unique=True)),
            ],
            options={
                'ordering': ['project__title', 'app_plugin__name', 'name'],
            },
        ),
        migrations.CreateModel(
            name='ProjectUserTag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(default='STARRED', help_text='Name of tag to be assigned', max_length=64)),
                ('omics_uuid', models.UUIDField(default=uuid.uuid4, help_text='ProjectUserTag Omics UUID', unique=True)),
            ],
            options={
                'ordering': ['project__title', 'user__username', 'name'],
            },
        ),
        migrations.CreateModel(
            name='Role',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Name of role', max_length=64, unique=True)),
                ('description', models.TextField(help_text='Role description')),
            ],
        ),
        migrations.CreateModel(
            name='RoleAssignment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('omics_uuid', models.UUIDField(default=uuid.uuid4, help_text='RoleAssignment Omics UUID', unique=True)),
                ('project', models.ForeignKey(help_text='Project in which role is assigned', on_delete=django.db.models.deletion.CASCADE, related_name='roles', to='projectroles.Project')),
                ('role', models.ForeignKey(help_text='Role to be assigned', on_delete=django.db.models.deletion.CASCADE, related_name='assignments', to='projectroles.Role')),
            ],
            options={
                'ordering': ['project__parent__title', 'project__title', 'role__name', 'user__username'],
            },
        ),
    ]



"""UI view tests for the tokens app"""

from django.contrib.messages import get_messages
from django.urls import reverse
from django.utils import timezone

from knox.models import AuthToken

from test_plus.test import TestCase

from tokens.views import TOKEN_CREATE_MSG, TOKEN_DELETE_MSG


class TestUserTokenListView(TestCase):
    """Tests for UserTokenListView"""

    def _make_token(self):
        self.tokens = [AuthToken.objects.create(self.user, None)]

    def setUp(self):
        self.user = self.make_user()
        self.url = reverse('tokens:list')

    def test_get(self):
        """Test UserTokenListView GET with a token"""
        self._make_token()
        with self.login(self.user):
            response = self.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['object_list']), 1)

    def test_get_no_tokens(self):
        """Test GET with no tokens"""
        with self.login(self.user):
            response = self.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['object_list']), 0)


class TestUserTokenCreateView(TestCase):
    """Tests for UserTokenCreateView"""

    def setUp(self):
        self.user = self.make_user()
        self.url = reverse('tokens:create')

    def test_get(self):
        """Test UserTokenCreateView GET"""
        with self.login(self.user):
            response = self.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertIsNotNone(response.context['form'])

    def test_post_no_ttl(self):
        """Test POST with TTL=0"""
        self.assertEqual(AuthToken.objects.count(), 0)
        with self.login(self.user):
            response = self.post(self.url, data={'ttl': 0})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            list(get_messages(response.wsgi_request))[0].message,
            TOKEN_CREATE_MSG,
        )
        self.assertEqual(AuthToken.objects.count(), 1)
        self.assertEqual(AuthToken.objects.first().expiry, None)

    def test_post_ttl(self):
        """Test POST with TTL != 0"""
        self.assertEqual(AuthToken.objects.count(), 0)
        with self.login(self.user):
            response = self.post(self.url, data={'ttl': 10})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            list(get_messages(response.wsgi_request))[0].message,
            TOKEN_CREATE_MSG,
        )
        self.assertEqual(AuthToken.objects.count(), 1)
        self.assertEqual(
            AuthToken.objects.first().expiry.replace(
                minute=0, second=0, microsecond=0
            ),
            (timezone.now() + timezone.timedelta(hours=10)).replace(
                minute=0, second=0, microsecond=0
            ),
        )


class TestUserTokenDeleteView(TestCase):
    """Tests for UserTokenDeleteView"""

    def setUp(self):
        self.user = self.make_user()
        self.token = AuthToken.objects.create(user=self.user)[0]
        self.url = reverse('tokens:delete', kwargs={'pk': self.token.pk})

    def test_get(self):
        """Test UserTokenDeleteView GET"""
        with self.login(self.user):
            response = self.get(self.url)
        self.assertEqual(response.status_code, 200)

    def test_post(self):
        """Test token deletion"""
        self.assertEqual(AuthToken.objects.count(), 1)
        with self.login(self.user):
            response = self.post(self.url)
            self.assertRedirects(
                response, reverse('tokens:list'), fetch_redirect_response=False
            )
        self.assertEqual(
            list(get_messages(response.wsgi_request))[0].message,
            TOKEN_DELETE_MSG,
        )
        self.assertEqual(AuthToken.objects.count(), 0)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2019-09-23 12:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0011_remove_indexes'),
    ]

    operations = [
        migrations.AddField(
            model_name='remotesite',
            name='user_display',
            field=models.BooleanField(default=True, help_text='RemoteSite visibility to users'),
        ),
        migrations.AlterField(
            model_name='remotesite',
            name='secret',
            field=models.CharField(help_text='Secret token for connecting to the source site', max_length=255, null=True),
        ),
    ]



"""Plugin point definitions and plugin API for apps based on projectroles"""

from django.conf import settings
from djangoplugins.point import PluginPoint


# Local costants
PLUGIN_TYPES = {
    'project_app': 'ProjectAppPluginPoint',
    'backend': 'BackendPluginPoint',
    'site_app': 'SiteAppPluginPoint',
}

# From djangoplugins
ENABLED = 0
DISABLED = 1
REMOVED = 2


# Plugin Mixins ----------------------------------------------------------------


class ProjectModifyPluginMixin:
    """
    Mixin for project plugin API extensions for additional actions to be
    performed for project and role modifications. Used if e.g. updating external
    resources based on SODAR Core projects.

    Add this into your project app or backend plugin if you want to implement
    additional modification features. It is not supported on site app plugins.
    """

    def perform_project_modify(
        self,
        project,
        action,
        project_settings,
        old_data=None,
        old_settings=None,
        request=None,
    ):
        """
        Perform additional actions to finalize project creation or update.

        :param project: Current project object (Project)
        :param action: Action to perform (CREATE or UPDATE)
        :param project_settings: Project app settings (dict)
        :param old_data: Old project data in case of an update (dict or None)
        :param old_settings: Old app settings in case of update (dict or None)
        :param request: Request object or None
        """
        # TODO: Implement this in your app plugin
        pass

    def revert_project_modify(
        self,
        project,
        action,
        project_settings,
        old_data=None,
        old_settings=None,
        request=None,
    ):
        """
        Revert project creation or update if errors have occurred in other apps.

        :param project: Current project object (Project)
        :param action: Action which was performed (CREATE or UPDATE)
        :param project_settings: Project app settings (dict)
        :param old_data: Old project data in case of update (dict or None)
        :param old_settings: Old app settings in case of update (dict or None)
        :param request: Request object or None
        """
        # TODO: Implement this in your app plugin
        pass

    def perform_role_modify(self, role_as, action, old_role=None, request=None):
        """
        Perform additional actions to finalize role assignment creation or
        update.

        :param role_as: RoleAssignment object
        :param action: Action to perform (CREATE or UPDATE)
        :param old_role: Role object for previous role in case of an update
        :param request: Request object or None
        """
        # TODO: Implement this in your app plugin
        pass

    def revert_role_modify(self, role_as, action, old_role=None, request=None):
        """
        Revert role assignment creation or update if errors have occurred in
        other apps.

        :param role_as: RoleAssignment object
        :param action: Action which was performed (CREATE or UPDATE)
        :param old_role: Role object for previous role in case of an update
        :param request: Request object or None
        """
        # TODO: Implement this in your app plugin
        pass

    def perform_role_delete(self, role_as, request=None):
        """
        Perform additional actions to finalize role assignment deletion.

        :param role_as: RoleAssignment object
        :param request: Request object or None
        """
        # TODO: Implement this in your app plugin
        pass

    def revert_role_delete(self, role_as, request=None):
        """
        Revert role assignment deletion deletion if errors have occurred in
        other apps.

        :param role_as: RoleAssignment object
        :param request: Request object or None
        """
        # TODO: Implement this in your app plugin
        pass

    def perform_owner_transfer(
        self, project, new_owner, old_owner, old_owner_role, request=None
    ):
        """
        Perform additional actions to finalize project ownership transfer.

        :param project: Project object
        :param new_owner: SODARUser object for new owner
        :param old_owner: SODARUser object for previous owner
        :param old_owner_role: Role object for new role of previous owner
        :param request: Request object or None
        """
        # TODO: Implement this in your app plugin
        pass

    def revert_owner_transfer(
        self, project, new_owner, old_owner, old_owner_role, request=None
    ):
        """
        Revert project ownership transfer if errors have occurred in other apps.

        :param project: Project object
        :param new_owner: SODARUser object for new owner
        :param old_owner: SODARUser object for previous owner
        :param old_owner_role: Role object for new role of previous owner
        :param request: Request object or None
        """
        # TODO: Implement this in your app plugin
        pass

    def perform_project_sync(self, project):
        """
        Synchronize existing projects to ensure related data exists when the
        syncmodifyapi management comment is called. Should mostly be used in
        development when the development databases have been e.g. modified or
        recreated.

        :param project: Current project object (Project)
        """
        # TODO: Implement this in your app plugin
        pass

    def perform_project_setting_update(
        self,
        plugin_name,
        setting_name,
        value,
        old_value,
        project=None,
        user=None,
    ):
        """
        Perform additional actions when updating a single app setting with
        PROJECT scope.

        :param plugin_name: Name of app plugin for the setting, "projectroles"
                            is used for projectroles settings (string)
        :param setting_name: Setting name (string)
        :param value: New value for setting
        :param old_value: Previous value for setting
        :param project: Project object or None
        :param user: User object or None
        """
        # TODO: Implement this in your app plugin
        pass

    def revert_project_setting_update(
        self,
        plugin_name,
        setting_name,
        value,
        old_value,
        project=None,
        user=None,
    ):
        """
        Revert updating a single app setting with PROJECT scope if errors have
        occurred in other apps.

        :param plugin_name: Name of app plugin for the setting, "projectroles"
                            is used for projectroles settings (string)
        :param setting_name: Setting name (string)
        :param value: New value for setting
        :param old_value: Previous value for setting
        :param project: Project object or None
        :param user: User object or None
        """
        # TODO: Implement this in your app plugin
        pass

    def perform_project_archive(
        self,
        project,
    ):
        """
        Perform additional actions to finalize project archiving or unarchiving.
        The state being applied can be derived from the project.archive attr.

        :param project: Project object (Project)
        """
        # TODO: Implement this in your app plugin
        pass

    def revert_project_archive(
        self,
        project,
    ):
        """
        Revert project archiving or unarchiving if errors have occurred in other
        apps. The state being originally set can be derived from the
        project.archive attr.

        :param project: Project object (Project)
        """
        # TODO: Implement this in your app plugin
        pass


# Plugin Points ----------------------------------------------------------------


class ProjectAppPluginPoint(PluginPoint):
    """Projectroles plugin point for registering project specific apps"""

    #: UI URLs
    urls = []

    #: App settings definition
    #:
    #: Example ::
    #:
    #:     app_settings = {
    #:         'example_setting': {
    #:             'scope': 'PROJECT',  # PROJECT/USER/SITE
    #:             'type': 'STRING',  # STRING/INTEGER/BOOLEAN
    #:             'default': 'example',
    #:             'label': 'Project setting',  # Optional, defaults to name/key
    #:             'placeholder': 'Enter example setting here',  # Optional
    #:             'description': 'Example project setting',  # Optional
    #:             'options': ['example', 'example2'],  # Optional, only for
    #:             settings of type STRING or INTEGER
    #:             'user_modifiable': True,  # Optional, show/hide in forms
    #:             'local': False,  # Optional, show/hide in forms on target
    #:             site, sync value from source to target site if false
    #:             'project_types': [PROJECT_TYPE_PROJECT],  # Optional, may
    #:             contain PROJECT_TYPE_CATEGORY and/or PROJECT_TYPE_PROJECT
    #:         }
    #:     }
    # TODO: Define project specific settings in your app plugin, example above
    app_settings = {}

    # DEPRECATED, will be removed in the next SODAR Core release
    project_settings = {}

    #: Iconify icon
    # TODO: Implement this in your app plugin
    icon = 'mdi:help-rhombus-outline'

    #: Entry point URL ID (must take project sodar_uuid as "project" argument)
    # TODO: Implement this in your app plugin
    entry_point_url_id = 'home'

    #: Description string
    # TODO: Implement this in your app plugin
    description = 'TODO: Write a description for your plugin'

    #: Required permission for accessing the app
    # TODO: Implement this in your app plugin (can be None)
    app_permission = None

    #: Enable or disable general search from project title bar
    # TODO: Implement this in your app plugin
    search_enable = False

    #: List of search object types for the app
    # TODO: Implement this in your app plugin
    search_types = []

    #: Search results template
    # TODO: Implement this in your app plugin
    search_template = None

    #: App card template for the project details page
    # TODO: Implement this in your app plugin
    details_template = None

    #: App card title for the project details page
    # TODO: Implement this in your app plugin (can be None)
    details_title = None

    #: Position in plugin ordering
    # TODO: Implement this in your app plugin (must be an integer)
    plugin_ordering = 50

    #: Optional custom project list column definition
    #:
    #: Example ::
    #:
    #:     project_list_columns = {
    #:         'column_id': {
    #:             'title': 'Column Title',
    #:             'width': 100,  # Desired width of column in pixels
    #:             'description': 'Description',  # Optional description string
    #:             'active': True,  # Boolean for whether the column is active
    #:             'ordering': 50,  # Integer for ordering the columns
    #:             'align': 'left'  # Alignment of content
    #:         }
    #:     }
    # TODO: Define project list column data in your app plugin (optional)
    project_list_columns = {}

    #: Display application for categories in addition to projects
    # TODO: Override this in your app plugin if needed
    category_enable = False

    #: Names of plugin specific Django settings to display in siteinfo
    # TODO: Override this in your app plugin if needed
    info_settings = []

    def get_object(self, model, uuid):
        """
        Return object based on a model class and the object's SODAR UUID.

        :param model: Object model class
        :param uuid: sodar_uuid of the referred object
        :return: Model object or None if not found
        :raise: NameError if model is not found
        """
        try:
            return model.objects.get(sodar_uuid=uuid)
        except model.DoesNotExist:
            return None

    def get_object_link(self, model_str, uuid):
        """
        Return URL referring to an object used by the app, along with a name to
        be shown to the user for linking.

        :param model_str: Object class (string)
        :param uuid: sodar_uuid of the referred object
        :return: PluginObjectLink or None if not found
        """
        obj = self.get_object(eval(model_str), uuid)
        if not obj:
            return None
        # TODO: Implement this in your app plugin
        return None

    def get_extra_data_link(self, _extra_data, _name):
        """Return a link for timeline label starting with 'extra-'"""
        # TODO: Implement this in your app plugin
        return None

    def search(self, search_terms, user, search_type=None, keywords=None):
        """
        Return app items based on one or more search terms, user, optional type
        and optional keywords.

        :param search_terms: Search terms to be joined with the OR operator
                             (list of strings)
        :param user: User object for user initiating the search
        :param search_type: String
        :param keywords: List (optional)
        :return: List of PluginSearchResult objects
        """
        # TODO: Implement this in your app plugin
        # TODO: Implement display of results in the app's search template
        return []

    def update_cache(self, name=None, project=None, user=None):
        """
        Update cached data for this app, limitable to item ID and/or project.

        :param name: Item name to limit update to (string, optional)
        :param project: Project object to limit update to (optional)
        :param user: User object to denote user triggering the update (optional)
        """
        # TODO: Implement this in your app plugin
        return None

    def get_statistics(self):
        """
        Return app statistics as a dict. Should take the form of
        {id: {label, value, url (optional), description (optional)}}.

        :return: Dict
        """
        # TODO: Implement this in your app plugin
        return {}

    def get_project_list_value(self, column_id, project, user):
        """
        Return a value for the optional additional project list column specific
        to a project.

        :param column_id: ID of the column (string)
        :param project: Project object
        :param user: User object (current user)
        :return: String (may contain HTML), integer or None
        """
        # TODO: Implement this in your app plugin (optional)
        return None

    def validate_form_app_settings(self, app_settings, project=None, user=None):
        """
        Validate app settings form data and return a dict of errors.

        :param app_settings: Dict of app settings
        :param project: Project object or None
        :param user: User object or None
        :return: dict in format of {setting_name: 'Error string'}
        """
        # TODO: Implement this in your app plugin (optional)
        return None


class BackendPluginPoint(PluginPoint):
    """Projectroles plugin point for registering backend apps"""

    #: Iconify icon
    # TODO: Implement this in your backend plugin
    icon = 'mdi:help-rhombus-outline'

    #: Description string
    # TODO: Implement this in your backend plugin
    description = 'TODO: Write a description for your plugin'

    #: URL of optional javascript file to be included
    # TODO: Implement this in your backend plugin if applicable
    javascript_url = None

    #: URL of optional css file to be included
    # TODO: Implement this in your backend plugin if applicable
    css_url = None

    #: Names of plugin specific Django settings to display in siteinfo
    # TODO: Override this in your app plugin if needed
    info_settings = []

    def get_api(self):
        """Return API entry point object."""
        # TODO: Implement this in your backend plugin
        raise NotImplementedError

    def get_statistics(self):
        """
        Return backend statistics as a dict. Should take the form of
        {id: {label, value, url (optional), description (optional)}}.

        :return: Dict
        """
        # TODO: Implement this in your backend plugin
        return {}

    def get_object(self, model, uuid):
        """
        Return object based on a model class and the object's SODAR UUID.

        :param model: Object model class
        :param uuid: sodar_uuid of the referred object
        :return: Model object or None if not found
        :raise: NameError if model is not found
        """
        try:
            return model.objects.get(sodar_uuid=uuid)
        except model.DoesNotExist:
            return None

    def get_object_link(self, model_str, uuid):
        """
        Return URL referring to an object used by the app, along with a name to
        be shown to the user for linking.

        :param model_str: Object class (string)
        :param uuid: sodar_uuid of the referred object
        :return: PluginObjectLink or None if not found
        """
        obj = self.get_object(eval(model_str), uuid)
        if not obj:
            return None
        # TODO: Implement this in your app plugin
        return None

    def get_extra_data_link(self, _extra_data, _name):
        """Return a link for timeline label starting with 'extra-'"""
        # TODO: Implement this in your app plugin
        return None


class SiteAppPluginPoint(PluginPoint):
    """Projectroles plugin point for registering site-wide apps"""

    #: Iconify icon
    # TODO: Implement this in your site app plugin
    icon = 'mdi:help-rhombus-outline'

    #: Description string
    # TODO: Implement this in your site app plugin
    description = 'TODO: Write a description for your plugin'

    #: Entry point URL ID
    # TODO: Implement this in your app plugin
    entry_point_url_id = 'home'

    #: Required permission for displaying the app
    # TODO: Implement this in your site app plugin (can be None)
    app_permission = None

    #: User settings definition
    #:
    #: Example ::
    #:
    #:     app_settings = {
    #:         'example_setting': {
    #:             'scope' : 'USER',  # always USER
    #:             'type': 'STRING',  # STRING/INTEGER/BOOLEAN
    #:             'default': 'example',
    #:             'placeholder': 'Enter example setting here',  # Optional
    #:             'description': 'Example user setting',  # Optional
    #:             'options': ['example', 'example2'],  # Optional, only for
    #:             settings of type STRING or INTEGER
    #:             'user_modifiable': True,  # Optional, show/hide in forms
    #:             'local': False,  # Optional, show/hide in forms on target
    #:             site
    #:         }
    #:     }
    # TODO: Define user specific settings in your app plugin, example above
    app_settings = {}

    #: List of names for plugin specific Django settings to display in siteinfo
    # TODO: Override this in your app plugin if needed
    info_settings = []

    def get_statistics(self):
        """
        Return app statistics as a dict. Should take the form of
        {id: {label, value, url (optional), description (optional)}}.

        :return: Dict
        """
        # TODO: Implement this in your app plugin
        return {}

    def get_messages(self, user=None):
        """
        Return a list of messages to be shown to users.

        :param user: User object (optional)
        :return: List of dicts or and empty list if no messages
        """
        '''
        Example of valid return data:
        return [{
            'content': 'Message content in here, can contain html',
            'color': 'info',        # Corresponds to bg-* in Bootstrap
            'dismissable': True     # False for non-dismissable
        }]
        '''
        # TODO: Implement this in your site app plugin
        return []

    def get_object(self, model, uuid):
        """
        Return object based on a model class and the object's SODAR UUID.

        :param model: Object model class
        :param uuid: sodar_uuid of the referred object
        :return: Model object or None if not found
        :raise: NameError if model is not found
        """
        try:
            return model.objects.get(sodar_uuid=uuid)
        except model.DoesNotExist:
            return None

    def get_object_link(self, model_str, uuid):
        """
        Return URL referring to an object used by the app, along with a name to
        be shown to the user for linking.

        :param model_str: Object class (string)
        :param uuid: sodar_uuid of the referred object
        :return: PluginObjectLink or None if not found
        """
        obj = self.get_object(eval(model_str), uuid)
        if not obj:
            return None
        # TODO: Implement this in your app plugin
        return None

    def get_extra_data_link(self, _extra_data, _name):
        """Return a link for timeline label starting with 'extra-'"""
        # TODO: Implement this in your app plugin
        return None

    def validate_form_app_settings(self, app_settings, user=None):
        """
        Validate app settings form data and return a dict of errors.

        :param app_settings: Dict of app settings
        :param user: User object or None
        :return: dict in format of {setting_name: 'Error string'}
        """
        # TODO: Implement this in your app plugin (optional)
        return None


# Data Classes -----------------------------------------------------------------


class PluginObjectLink:
    """
    Class representing a hyperlink to an object used by the app. Expected to be
    returned from get_object_link() implementations.
    """

    #: URL to the object (string)
    url = None
    #: Name of the object to be displayed in link (string, formerly "label")
    name = None
    #: Open the link in a blank browser tab (boolean, default=False)
    blank = False

    def __init__(self, url, name, blank=False):
        """
        Initialize PluginObjectLink.

        :param url: URL to the object (string)
        :param name: Name of the object to be displayed in link (string,
                     formerly "label")
        :param blank: Open the link in a blank browser tab (boolean,
                      default=False)
        """
        self.url = url
        self.name = name
        self.blank = blank


class PluginSearchResult:
    """
    Class representing a list of search results from a specific plugin for one
    or more search types. Expected to be returned from search() implementations.
    """

    #: Category of the result set, used in templates (string)
    category = None
    #: Title to be displayed for this set of search results in the UI (string)
    title = None
    #: List of one or more search type keywords for these results
    search_types = []
    #: List or QuerySet of result objects
    items = []

    def __init__(self, category, title, search_types, items):
        """
        Initialize PluginSearchResult.

        :param category: Category of the result set, used in templates (string)
        :param title: Title to be displayed for this set of search results in
                      the UI (string)
        :param search_types: List of one or more search type keywords for the
                             results
        :param items: List or QuerySet of result objects
        """
        self.category = category
        self.title = title
        self.search_types = search_types
        if not isinstance(search_types, list) or len(search_types) < 1:
            raise ValueError(
                'At least one type keyword must be provided in search_types'
            )
        self.items = items


# Plugin API -------------------------------------------------------------------


def get_active_plugins(plugin_type='project_app', custom_order=False):
    """
    Return active plugins of a specific type.

    :param plugin_type: "project_app", "site_app" or "backend" (string)
    :param custom_order: Order by plugin_ordering for project apps (boolean)
    :return: List or None
    :raise: ValueError if plugin_type is not recognized
    """
    if plugin_type not in PLUGIN_TYPES.keys():
        raise ValueError(
            'Invalid value for plugin_type. Accepted values: {}'.format(
                ', '.join(PLUGIN_TYPES.keys())
            )
        )
    plugins = eval(PLUGIN_TYPES[plugin_type]).get_plugins()
    if plugins:
        ret = [
            p
            for p in plugins
            if (
                p.is_active()
                and (
                    plugin_type in ['project_app', 'site_app']
                    or p.name in settings.ENABLED_BACKEND_PLUGINS
                )
            )
        ]
        return sorted(
            ret,
            key=lambda x: (
                x.plugin_ordering
                if custom_order and plugin_type == 'project_app'
                else x.name
            ),
        )
    return None


def change_plugin_status(name, status, plugin_type='app'):
    """
    Change the status of a selected plugin in the database.

    :param name: Plugin name (string)
    :param status: Status (int, see djangoplugins)
    :param plugin_type: Type of plugin ("app", "backend" or "site")
    :raise: ValueError if plugin_type is invalid or plugin with name not found
    """
    # NOTE: Used to forge plugin to a specific status for e.g. testing
    if plugin_type == 'app':
        plugin = ProjectAppPluginPoint.get_plugin(name)
    elif plugin_type == 'backend':
        plugin = BackendPluginPoint.get_plugin(name)
    elif plugin_type == 'site':
        plugin = SiteAppPluginPoint.get_plugin(name)
    else:
        raise ValueError('Invalid plugin_type: "{}"'.format(plugin_type))
    if not plugin:
        raise ValueError(
            'Plugin of type "{}" not found with name "{}"'.format(
                plugin_type, name
            )
        )
    plugin = plugin.get_model()
    plugin.status = status
    plugin.save()


def get_app_plugin(plugin_name, plugin_type=None):
    """
    Return active app plugin.

    :param plugin_name: Plugin name (string)
    :param plugin_type: Plugin type (string or None for all types)
    :return: Plugin object or None if not found
    """
    if plugin_type:
        plugin_types = [PLUGIN_TYPES[plugin_type]]
    else:
        plugin_types = PLUGIN_TYPES.values()
    for t in plugin_types:
        try:
            return eval(t).get_plugin(plugin_name)
        except Exception:
            pass


def get_backend_api(plugin_name, force=False, **kwargs):
    """
    Return backend API object.
    NOTE: May raise an exception from plugin.get_api().

    :param plugin_name: Plugin name (string)
    :param force: Return plugin regardless of status in ENABLED_BACKEND_PLUGINS
    :param kwargs: Optional kwargs for API
    :return: Plugin object or None if not found
    """
    if plugin_name in settings.ENABLED_BACKEND_PLUGINS or force:
        try:
            plugin = BackendPluginPoint.get_plugin(plugin_name)
        except BackendPluginPoint.DoesNotExist:
            return None
        return plugin.get_api(**kwargs) if plugin.is_active() else None


# Plugins within projectroles --------------------------------------------------


class RemoteSiteAppPlugin(SiteAppPluginPoint):
    """Site plugin for remote site and project management"""

    #: Name (used as plugin ID)
    name = 'remotesites'

    #: Title (used in templates)
    title = 'Remote Site Access'

    #: UI URLs
    urls = []

    #: Iconify icon
    icon = 'mdi:cloud-sync'

    #: Description string
    description = 'Management of remote SODAR sites and remote project access'

    #: Entry point URL ID
    entry_point_url_id = 'projectroles:remote_sites'

    #: Required permission for displaying the app
    app_permission = 'userprofile.update_remote'



# Generated by Django 3.2.12 on 2022-04-07 15:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('timeline', '0007_allow_null_project'),
    ]

    operations = [
        migrations.AddField(
            model_name='projectevent',
            name='plugin',
            field=models.CharField(blank=True, help_text='Plugin to which the event is related (optional, if unset plugin with the name of the app is assumed)', max_length=255, null=True),
        ),
    ]



"""Tests for UI view permissions in the tokens app"""

from django.test import override_settings
from django.urls import reverse

from knox.models import AuthToken

# Projectroles dependency
from projectroles.tests.test_permissions import SiteAppPermissionTestBase


class TestTokenPermissions(SiteAppPermissionTestBase):
    """Tests for token view permissions"""

    def test_get_list(self):
        """Test tUserTokenListView GET"""
        url = reverse('tokens:list')
        self.assert_response(url, [self.superuser, self.regular_user], 200)
        self.assert_response(url, self.anonymous, 302)

    @override_settings(PROJECTROLES_ALLOW_ANONYMOUS=True)
    def test_get_list_anon(self):
        """Test UserTokenListView GET with anonymous access"""
        url = reverse('tokens:list')
        self.assert_response(url, [self.superuser, self.regular_user], 200)
        self.assert_response(url, self.anonymous, 302)

    def test_get_create(self):
        """Test UserTokenCreateView GET"""
        url = reverse('tokens:create')
        self.assert_response(url, [self.superuser, self.regular_user], 200)
        self.assert_response(url, self.anonymous, 302)

    @override_settings(PROJECTROLES_ALLOW_ANONYMOUS=True)
    def test_get_create_anon(self):
        """Test UserTokenCreateView GET with anonymous access"""
        url = reverse('tokens:create')
        self.assert_response(url, [self.superuser, self.regular_user], 200)
        self.assert_response(url, self.anonymous, 302)

    def test_get_delete(self):
        """Test UserTokenDeleteView GET"""
        token = AuthToken.objects.create(self.regular_user, None)
        url = reverse('tokens:delete', kwargs={'pk': token[0].pk})
        self.assert_response(url, self.regular_user, 200)
        self.assert_response(url, self.anonymous, 302)



# Projectroles dependency
from projectroles.plugins import SiteAppPluginPoint

from userprofile.urls import urlpatterns


class SiteAppPlugin(SiteAppPluginPoint):
    """Projectroles plugin for registering the app"""

    #: Name (used as plugin ID)
    name = 'userprofile'

    #: Title (used in templates)
    title = 'User Profile'

    #: UI URLs
    urls = urlpatterns

    #: Iconify icon
    icon = 'mdi:account-details'

    #: Description string
    description = 'Project User Profile'

    #: Entry point URL ID
    entry_point_url_id = 'userprofile:detail'

    #: Required permission for displaying the app
    app_permission = 'userprofile.view_detail'

    app_settings = {
        'enable_project_uuid_copy': {
            'scope': 'USER',
            'type': 'BOOLEAN',
            'label': 'Display project UUID copying link',
            'default': False,
            'user_modifiable': True,
        }
    }

    def get_messages(self, user=None):
        """
        Return a list of messages to be shown to users.
        :param user: User object (optional)
        :return: List of dicts or and empty list if no messages
        """
        messages = []
        return messages



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-10-23 10:27
from __future__ import unicode_literals

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filesfolders', '0003_rename_uuid'),
    ]

    operations = [
        migrations.AlterField(
            model_name='file',
            name='sodar_uuid',
            field=models.UUIDField(default=uuid.uuid4, help_text='Filesfolders SODAR UUID', unique=True),
        ),
        migrations.AlterField(
            model_name='folder',
            name='sodar_uuid',
            field=models.UUIDField(default=uuid.uuid4, help_text='Filesfolders SODAR UUID', unique=True),
        ),
        migrations.AlterField(
            model_name='hyperlink',
            name='sodar_uuid',
            field=models.UUIDField(default=uuid.uuid4, help_text='Filesfolders SODAR UUID', unique=True),
        ),
    ]



import random
import string

from django.conf import settings
from django.urls import reverse
from django.utils import timezone

from projectroles.plugins import get_active_plugins
from projectroles.models import SODAR_CONSTANTS

# Settings
SECRET_LENGTH = getattr(settings, 'PROJECTROLES_SECRET_LENGTH', 32)
INVITE_EXPIRY_DAYS = settings.PROJECTROLES_INVITE_EXPIRY_DAYS

# SODAR constants
PROJECT_TYPE_PROJECT = SODAR_CONSTANTS['PROJECT_TYPE_PROJECT']
PROJECT_TYPE_CATEGORY = SODAR_CONSTANTS['PROJECT_TYPE_CATEGORY']

# Local constants
SIDEBAR_ICON_MIN_SIZE = 18
SIDEBAR_ICON_MAX_SIZE = 42
ROLE_URLS = [
    'roles',
    'role_create',
    'role_update',
    'role_delete',
    'invites',
    'invite_create',
    'invite_resend',
    'invite_revoke',
]


def get_display_name(key, title=False, count=1, plural=False):
    """
    Return display name from SODAR_CONSTANTS.

    :param key: Key in SODAR_CONSTANTS['DISPLAY_NAMES'] to return (string)
    :param title: Return name in title case if true (boolean, optional)
    :param count: Item count for returning plural form, overrides plural=False
                  if not 1 (int, optional)
    :param plural: Return plural form if True, overrides count != 1 if True
                   (boolean, optional)
    :return: String
    """
    ret = SODAR_CONSTANTS['DISPLAY_NAMES'][key][
        'plural' if count != 1 or plural else 'default'
    ]
    return ret.lower() if not title else ret.title()


def get_user_display_name(user, inc_user=False):
    """
    Return full name of user for displaying.

    :param user: User object
    :param inc_user: Include user name if true (boolean)
    :return: String
    """
    if user.name != '':
        return user.name + (' (' + user.username + ')' if inc_user else '')
    # If full name can't be found, return username
    return user.username


def build_secret(length=SECRET_LENGTH):
    """
    Return secret string for e.g. public URLs.

    :param length: Length of string if specified, default value from settings
    :return: Randomized secret (string)
    """
    length = int(length) if int(length) <= 255 else 255
    return ''.join(
        random.SystemRandom().choice(string.ascii_lowercase + string.digits)
        for _ in range(length)
    )


def build_invite_url(invite, request):
    """
    Return invite URL for a project invitation.

    :param invite: ProjectInvite object
    :param request: HTTP request
    :return: URL (string)
    """
    return request.build_absolute_uri(
        reverse('projectroles:invite_accept', kwargs={'secret': invite.secret})
    )


def get_expiry_date():
    """
    Return expiry date based on current date + INVITE_EXPIRY_DAYS

    :return: DateTime object
    """
    return timezone.now() + timezone.timedelta(days=INVITE_EXPIRY_DAYS)


def get_app_names():
    """Return list of names for locally installed non-django apps"""
    ret = []
    for a in settings.INSTALLED_APPS:
        s = a.split('.')
        if s[0] not in ['django', settings.SITE_PACKAGE]:
            if len(s) > 1 and 'apps' in s:
                ret.append('.'.join(s[0 : s.index('apps')]))
            else:
                ret.append(s[0])
    return sorted(ret)


class AppLinkContent:
    """Class for generating application links for the UI"""

    def _is_active_projectroles(
        self, app_name=None, url_name=None, link_names=None
    ):
        """Check if current URL is active under the projectroles app."""
        if not app_name and not url_name:
            return False
        # HACK: Avoid circular import
        from projectroles.urls import urlpatterns

        if app_name != 'projectroles':
            return False
        return url_name in [u.name for u in urlpatterns] and (
            not link_names or url_name in link_names
        )

    def _is_active_plugin(self, app_plugin, app_name=None, url_name=None):
        """
        Check if current URL is active for a specific app plugin.
        """
        if not app_name and not url_name:
            return False
        if app_plugin.name.startswith(app_name) and (
            not url_name
            or url_name in [u.name for u in getattr(app_plugin, 'urls', [])]
        ):
            return True
        # HACK for remote site views, see issue #1336
        if (
            app_name == 'projectroles'
            and app_plugin.name == 'remotesites'
            and url_name.startswith('remote_')
        ):
            return True
        return False

    def _is_app_visible(self, plugin, project, user):
        """Check if app should be visible for user in a specific project."""
        can_view_app = user.has_perm(plugin.app_permission, project)
        app_hidden = False
        if plugin.name in getattr(
            settings, 'PROJECTROLES_HIDE_PROJECT_APPS', []
        ):
            app_hidden = True
        if (
            can_view_app
            and not app_hidden
            and (project.type == PROJECT_TYPE_PROJECT or plugin.category_enable)
        ):
            return True
        return False

    def _allow_project_creation(self):
        """Check whether creating a project is allowed on the site."""
        if (
            settings.PROJECTROLES_SITE_MODE
            == SODAR_CONSTANTS['SITE_MODE_TARGET']
            and not settings.PROJECTROLES_TARGET_CREATE
        ):
            return False
        return True

    def get_project_app_links(
        self, user, project=None, app_name=None, url_name=None
    ):
        """Return project app links based on the current project and user."""
        ret = []
        # Add project related links
        if project:
            project_display_name = get_display_name(project.type, title=True)
            # Add project overview link
            ret.append(
                {
                    'name': 'project-detail',
                    'url': reverse(
                        'projectroles:detail',
                        kwargs={'project': project.sodar_uuid},
                    ),
                    'label': f'{project_display_name} Overview',
                    'icon': (
                        'mdi:rhombus-split'
                        if project.type == PROJECT_TYPE_CATEGORY
                        else 'mdi:cube'
                    ),
                    'active': self._is_active_projectroles(
                        link_names=['detail'],
                        app_name=app_name,
                        url_name=url_name,
                    ),
                }
            )
            # Add app plugins links
            app_plugins = get_active_plugins()
            for plugin in app_plugins:
                if self._is_app_visible(plugin, project, user):
                    ret.append(
                        {
                            'name': "app-plugin-" + plugin.name,
                            'url': reverse(
                                plugin.entry_point_url_id,
                                kwargs={'project': project.sodar_uuid},
                            ),
                            'label': ' '.join(plugin.title.split(' ')),
                            'icon': plugin.icon,
                            'active': self._is_active_plugin(
                                plugin, app_name=app_name, url_name=url_name
                            ),
                        }
                    )
            # Add role editing link
            if user.has_perm('projectroles.view_project_roles', project):
                ret.append(
                    {
                        'name': 'project-roles',
                        'url': reverse(
                            'projectroles:roles',
                            kwargs={'project': project.sodar_uuid},
                        ),
                        'label': 'Members',
                        'icon': 'mdi:account-multiple',
                        'active': self._is_active_projectroles(
                            link_names=ROLE_URLS,
                            app_name=app_name,
                            url_name=url_name,
                        ),
                    }
                )
            # Add project update link
            if user.has_perm('projectroles.update_project', project):
                ret.append(
                    {
                        'name': 'project-update',
                        'url': reverse(
                            'projectroles:update',
                            kwargs={'project': project.sodar_uuid},
                        ),
                        'label': f'Update {project_display_name}',
                        'icon': 'mdi:lead-pencil',
                        'active': self._is_active_projectroles(
                            link_names=['update'],
                            app_name=app_name,
                            url_name=url_name,
                        ),
                    }
                )

        # Add project and category creation links
        if (
            project
            and project.type == PROJECT_TYPE_CATEGORY
            and user.has_perm('projectroles.create_project', project)
            and self._allow_project_creation()
            and not project.is_remote()
        ):
            ret.append(
                {
                    'name': 'project-create',
                    'url': reverse(
                        'projectroles:create',
                        kwargs={'project': project.sodar_uuid},
                    ),
                    'label': 'Create '
                    f'{get_display_name(PROJECT_TYPE_PROJECT, title=True)} '
                    f'or {get_display_name(PROJECT_TYPE_CATEGORY, title=True)}',
                    'icon': 'mdi:plus-thick',
                    'active': self._is_active_projectroles(
                        link_names=['create'],
                        app_name=app_name,
                        url_name=url_name,
                    ),
                }
            )
        elif (
            getattr(settings, 'PROJECTROLES_DISABLE_CATEGORIES', False)
            and user.is_superuser
        ):
            ret.append(
                {
                    'name': 'project-create',
                    'url': reverse('projectroles:create'),
                    'label': 'Create '
                    f'{get_display_name(PROJECT_TYPE_PROJECT, title=True)}',
                    'icon': 'mdi:plus-thick',
                    'active': self._is_active_projectroles(
                        link_names=['create'],
                        app_name=app_name,
                        url_name=url_name,
                    ),
                }
            )
        elif (
            (url_name == 'home' or app_name == 'projectroles' and not project)
            and user.has_perm('projectroles.create_project')
            and self._allow_project_creation()
        ):
            ret.append(
                {
                    'name': 'home-project-create',
                    'url': reverse('projectroles:create'),
                    'label': 'Create '
                    f'{get_display_name(PROJECT_TYPE_CATEGORY, title=True)}',
                    'icon': 'mdi:plus-thick',
                    'active': self._is_active_projectroles(
                        link_names=['create'],
                        app_name=app_name,
                        url_name=url_name,
                    ),
                }
            )
        return ret

    def get_user_links(self, user, app_name=None, url_name=None):
        """Return user links for the user dropdown"""
        ret = []
        # Add site-wide apps links
        site_apps = get_active_plugins('site_app')
        for app in site_apps:
            if not app.app_permission or user.has_perm(app.app_permission):
                ret.append(
                    {
                        'name': app.name,
                        'url': reverse(app.entry_point_url_id),
                        'label': app.title,
                        'icon': app.icon,
                        'active': self._is_active_plugin(
                            app, app_name=app_name, url_name=url_name
                        ),
                    }
                )
        # Add admin link
        if user.is_superuser:
            ret.append(
                {
                    'name': 'admin',
                    'url': reverse('admin:index'),
                    'label': 'Django Admin',
                    'icon': 'mdi:cogs',
                    'active': False,
                }
            )
        # Add log out / sign in link
        if user.is_authenticated:
            ret.append(
                {
                    'name': 'sign-out',
                    'url': reverse('logout'),
                    'label': 'Logout',
                    'icon': 'mdi:logout-variant',
                    'active': False,
                }
            )
        elif not getattr(settings, 'PROJECTROLES_KIOSK_MODE', False):
            ret.append(
                {
                    'name': 'sign-in',
                    'url': reverse('login'),
                    'label': 'Login',
                    'icon': 'mdi:login-variant',
                    'active': False,
                }
            )
        return ret



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-10-26 12:16
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [("bgjobs", "0001_initial")]

    operations = [
        migrations.CreateModel(
            name="BackgroundJobLogEntry",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "date_created",
                    models.DateTimeField(auto_now_add=True, help_text="DateTime of creation"),
                ),
                (
                    "level",
                    models.CharField(
                        choices=[
                            ("debug", "debug"),
                            ("info", "info"),
                            ("warning", "warning"),
                            ("error", "error"),
                        ],
                        help_text="Level of log entry",
                        max_length=50,
                    ),
                ),
                ("message", models.TextField(help_text="Log level's message")),
                (
                    "job",
                    models.ForeignKey(
                        help_text="Owning background job",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="log_entries",
                        to="bgjobs.BackgroundJob",
                    ),
                ),
            ],
            options={"ordering": ["date_created"]},
        )
    ]



# Generated by Django 3.2.18 on 2023-03-27 15:59

from django.db import migrations


def populate_finder_role(apps, schema_editor):
    Role = apps.get_model('projectroles', 'Role')
    role = Role(
        name='project finder',
        rank=50,
        project_types=['CATEGORY'],
        description='The project finder is able to see certain details of all '
        'categories and projects under a category for this role is given. '
        'They will not have access to the apps or data of those subprojects or '
        'categories until assigned a higher role. This role can only be '
        'assigned for categories.'
    )
    role.save()


def reverse_finder_role(apps, schema_editor):
    Role = apps.get_model('projectroles', 'Role')
    role = Role.objects.filter(name='project finder').first()
    if role:
        role.delete()


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0027_role_project_type'),
    ]

    operations = [
        migrations.RunPython(
            populate_finder_role, reverse_code=reverse_finder_role
        )
    ]



# Generated by Django 4.2.11 on 2024-06-11 14:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("projectroles", "0030_populate_sodaruseradditionalemail"),
    ]

    operations = [
        migrations.AddField(
            model_name="remotesite",
            name="owner_modifiable",
            field=models.BooleanField(
                default=True,
                help_text="Allow owners and delegates to modify project access for this site",
            ),
        ),
        migrations.AlterField(
            model_name="remotesite",
            name="user_display",
            field=models.BooleanField(default=True, help_text="Display site to users"),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-09-20 15:29
from __future__ import unicode_literals

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('timeline', '0003_rename_uuid'),
    ]

    operations = [
        migrations.AlterField(
            model_name='projectevent',
            name='sodar_uuid',
            field=models.UUIDField(default=uuid.uuid4, help_text='Event SODAR UUID', unique=True),
        ),
        migrations.AlterField(
            model_name='projecteventobjectref',
            name='object_uuid',
            field=models.UUIDField(blank=True, help_text='Object SODAR UUID', null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2021-01-29 08:16
from __future__ import unicode_literals

from django.db import migrations, models


def get_project_parents(obj):
    if not obj.parent:
        return None
    ret = []
    parent = obj.parent
    while parent:
        ret.append(parent)
        parent = parent.parent
    return reversed(ret)


def get_project_full_title(obj):
    parents = get_project_parents(obj)
    ret = ' / '.join([p.title for p in parents]) + ' / ' if parents else ''
    ret += obj.title
    return ret


def populate_full_title(apps, schema_editor):
    """Populate the new full_title field in the Project model"""
    Project = apps.get_model('projectroles', 'Project')
    for project in Project.objects.all():
        project.full_title = get_project_full_title(project)
        project.save()


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0016_app_plugin_field_none'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='full_title',
            field=models.CharField(help_text='Full project title with parent path (auto-generated)', max_length=4096, null=True),
        ),
        migrations.RunPython(
            populate_full_title,
            reverse_code=migrations.RunPython.noop
        ),
    ]



from django.db import migrations

from projectroles.models import PROJECT_TAG_STARRED


def migrate_stars(apps, schema_editor):
    """Create project_star AppSettings"""
    ProjectUserTag = apps.get_model('projectroles', 'ProjectUserTag')
    AppSetting = apps.get_model('projectroles', 'AppSetting')
    for tag in ProjectUserTag.objects.filter(name=PROJECT_TAG_STARRED):
        AppSetting.objects.get_or_create(
            project=tag.project,
            user=tag.user,
            value=True,
            name='project_star',
        )


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0023_project_archive'),
    ]

    operations = [
        migrations.RunPython(migrate_stars, reverse_code=migrations.RunPython.noop),
    ]
